<TeXmacs|0.3.5.14>

<style|<tuple|article|allouche>>

<\body>
  \;

  <\expand|make-title>
    <title|<TeXmacs> notes>

    <author|David Allouche>
  </expand>

  <\rsection|About this document>
    <\description>
      <expand|item*|Author:>David Allouche

      <expand|item*|<apply|TeXmacs>>0.3.0-3

      <expand|item*|Version:><with|font shape|slanted|0.1>

      <expand|item*|Distribution:>GNU General Public License

      <expand|item*|Last modification:>October 25, 2000
    </description>

    This document describes conventions, modules, classes, etc. used in
    <apply|TeXmacs>. It supposes the reader is fluent in <value|c> and
    <value|c++> programming and have a good understanding of object oriented
    modeling concepts. It focuses on interfaces and do not discuss low level
    parts of the program that are completely transparent to the reader of its
    source, like the memory allocation mechanism.
  </rsection>

  <\rsection|Conventions>
    <\rsection|Parameters <var|<with|mode|math|<left|langle><with|mode|text|n\
    ame<with|mode|math|<right|rangle>>>>2>>
      Many method parameters are named using a generic
      <var|<with|mode|math|<left|langle>>name<with|mode|math|<right|rangle>>>
      and the letter 2. For example: <var|env2> (for an environment),
      <var|rep2> (for a memory representation), <var|t2> (for a tree)...

      This naming convention is used to avoid name conflicts with an object
      member which is named directly from the base
      <var|<with|mode|math|<left|langle><with|mode|text|name<with|mode|math|<\
      right|rangle>>>>> in the same class.

      The programmer could have used the <var|this> variable, or the
      <var|the<with|mode|math|<left|langle><with|mode|text|name<with|mode|mat\
      h|<right|rangle>>>>> convention for data members. This is just a matter
      of programming style.
    </rsection>

    <\rsection|Modules>
      The source code contains many preprocessor directives starting with
      <cpp|#module>, <cpp|#endmodule> and <cpp|#import>. These directives are
      extensions implemented by the <with|font family|tt|gencc> preprocessor.
      This preprocessor provides an extended inclusion mechanism, automatic
      dependencies resolution, and a build system.

      To read the source you only need to know about the inclusion mechanism:

      <\itemize>
        <item>The inclusion unit is the <with|font shape|italic|module>. A
        module is an arbitrary piece of program. A single file can contain
        several modules.

        <item>A module starts with <cpp|#module>
        <module|<with|mode|math|<left|langle>>name<with|mode|math|<right|rang\
        le>>> and ends with <cpp|#endmodule>.

        <item>A module is included in another file with the preprocessor
        directive <cpp|#import> <module|<with|mode|math|<left|langle>>name<wi\
        th|mode|math|<right|rangle>>>. The inclusion just copies the text
        content of the module at the place of the <cpp|#import> directive,
        just like <cpp|#include> copies entire files.

        <item>A module can have parameters.

        A parameterized module begins with <cpp|#module>
        <module|<with|mode|math|<left|langle>>name<with|mode|math|<right|rang\
        le>>> (<with|mode|math|<left|langle>>p1<with|mode|math|<right|rangle>\
        ><with|mode|math|<left|[><with|mode|text|,
        <with|mode|math|<left|langle>>p2<with|mode|math|<right|rangle>>><righ\
        t|]>><with|mode|math|\<cdots\>>) and ends with <cpp|#endmodule>.

        <item>A parameterized module is included with <cpp|#import>
        <module|<with|mode|math|<left|langle>>name<with|mode|math|<right|rang\
        le>>> (<with|mode|math|<left|langle>>v1<with|mode|math|<right|rangle>\
        ><with|mode|math|<left|[><with|mode|text|,
        <with|mode|math|<left|langle>>v2<with|mode|math|<right|rangle>>><righ\
        t|]>><with|mode|math|\<cdots\>>). Inclusion of a parameterized module
        is done after substitution, by the parameters values, in the module
        contents, of the tokens matching a parameter name.
      </itemize>
    </rsection>

    <\rsection|Module conventions>
      Modules can be used in two ways. Either as fragments of class or as
      more conventional compilation units. The ``fragment of class'' approach
      is used a lot in the implementation of the reference counting. The
      ``compilation unit'' approch is used in all the program.

      Either way, most modules come in pairs, providing associated
      declarations and definitions.

      <\enumerate>
        <item><module|<with|mode|math|<left|langle>>module<with|mode|math|<ri\
        ght|rangle>>> provides method prototypes et data member declarations
        (``fragment of class'' way) or typedef's, class and function
        declarations. In one phrase, anything that does not produce any
        object code. In one word, declarations.

        <item><module|code_<with|mode|math|<left|langle><with|mode|text|modul\
        e><right|rangle>>> provides definitions for the declarations in
        <module|<with|mode|math|<left|langle>>module<with|mode|math|<right|ra\
        ngle>>>, and is meant to be included in <file|*.gen.cc> files.
      </enumerate>
    </rsection>
  </rsection>

  <\rsection|Reference counting>
    <\rsection|What is reference counting>
      Reference counting is a simple way to implement automatic disposal of
      dynamically allocated memory. The idea is to have every object (aka.
      memory block) keep a count of the existing pointers to it. When the
      count reaches zero, the object is no longer referenced by any pointer
      and, thus, can no longer be used, and it automagically becomes free
      memory.

      The main problem of reference couting is that cyclic structures may
      never be freed. If two objects contain a reference to one another, they
      will keep a non-zero reference count even if they are no longer
      referenced from anywhere else, and thus can no longer be used. This
      problem is not fatal, since it can be solved by a careful
      implementation of graph-like object structures. If you restrain
      yourself to tree-like structures you should never run into trouble.

      Essentially, when you need to put a cycle in your object structure, you
      must also define a facade object for the substructre requiring a cycle.
      If this facade object is not referenced from inside the substructure,
      its reference couting will be effective. The destructor of the facade
      object must then explicitely break the cycles by setting some reference
      variables (also called counted pointers) to <const|NULL>.
    </rsection>

    <\rsection|Reference counting in <apply|TeXmacs>>
      <with|font family|ss|File: <file|Basic/basic.gen.h>>

      In <apply|TeXmacs>, reference counting is implemented using six modules
      pairs defined in <file|Basic/basic.gen.h>. The prototype modules are
      <module|concrete>, <module|abstract>, <module|concrete_null>,
      <module|abstract_null>, <module|extend>, <module|extend_null>. This
      sums up to 75 lines of cleverly simple code, but needs quite a bit of
      explanation for who is not used to reference counting.

      Each of the first four modules pairs are used to give reference
      counting and pointer semantics to a class <type|T><type|>. They take
      two parameters:

      <\description>
        <expand|item*|<var|T>>Class in which the module is imported. It will
        be referred to as a the <em|reference counting>, or the <em|counted
        pointer> class. Usually this class has the basic name (like
        <type|tree>), because it is the type that will be used in most client
        code.

        <expand|item*|<var|R>>Corresponding <with|font shape|italic|memory
        representation> class. Usually this class is named using the pattern
        <type|<with|mode|math|<left|langle><with|mode|text|>T<right|rangle>>_\
        rep>. (like <type|tree_rep>)
      </description>

      When defining a counted pointer class, a prototype module must be
      imported at the beginning of the class declaration (or anywhere else
      where the current visibility is <with|font family|ss|private>). The
      prototype module will set the visibility to <with|font
      family|ss|public>, so if the subsequent member declarations must be
      <with|font family|ss|private>, the visibility should be set back
      explicitly. However, you should not need to have any restricted member
      in a reference counting class.

      The code module associated to the the counted pointer class must also
      import the corresponding code module with the same parameters

      <\rsection|Classes conventions>
        The class <type|T> will be the one that is used to make variables. It
        will define the constructors and most methods.

        The class <type|R>, usually named
        <type|<with|mode|math|<left|langle>>T<with|mode|math|<right|rangle>>_\
        rep>, will define the actual data members and the <with|font
        shape|italic|terminal constructors> that are used by constructors
        defined in <type|T>. It is usually a specialization of
        <type|><type|concrete_struct> or <type|abstract_struct>. It should
        not be used for variables since this would make an exception to the
        reference counting system.

        The benefit of deriving all memory representation classes from
        <type|concrete_struct> or <type|abstract_struct>, apart from correct
        encapsulation of the name of the reference counter data member, is to
        allow debugging of the reference counting system.

        Most classes <type|T> overload the <funcproto|T|copy|<param|T|>>
        function to create a reference to a new representation.
      </rsection>

      <\rsection|Module pair <module|concrete>>
        This module pair must be used in pointer counting classes which do
        not implement any abstract class and which are not extended by any
        other class.

        <expand|paragraph|Declaration module>

        <\code>
          <cpp|#module> <module|concrete> (<var|T>, <var|R>)

          <indent|<type|R<apply|oper|*>> <var|rep>;<expand|//|<htab|5mm>memor\
          y representation>>

          <keyw|public>:

          <\indent>
            <keyw|inline> <funcproto||T|<param|<keyw|const>
            <var|T><apply|oper|&>|x>>;<expand|//|increase the reference
            count>

            <keyw|inline> <funcproto||<apply|oper|~>T|>;<expand|//|decrease
            the reference count>

            <keyw|inline> <operproto|R<apply|oper|*>|-\<gtr\>|>;<expand|//|ac\
            cess to data members>

            <keyw|inline> <operproto|T|=|<param|T|x>>;<expand|//|increase the
            reference count>
          </indent>

          <cpp|#endmodule><expand|//|end of module <module|concrete>>
        </code>

        This is the simplest module pair. It implements the basic pointer
        counting functionnality.

        The <var|rep> data member must point to a class which defines the
        <var|ref_count> data member, this is the reference counter.
      </rsection>

      <\rsection|Module pair <module|abstract>>
        This module pair must be used in classes which do not derive of any
        class, but do have derived classes.

        <expand|paragraph|Declaration module>

        <\code>
          <cpp|#module> <module|abstract> (<var|T>, <var|R>)

          <cpp|#import> <module|concrete> (<var|T>, <var|R>)

          <indent|<keyw|inline> <funcproto||T|<param|R<apply|oper|*>|rep>>;<e\
          xpand|//|increase the reference count>>

          <cpp|#endmodule>
        </code>

        These modules import the pointer counting functionnality from the
        <module|concrete> module pair. They also implement an additional
        constructor that is used to implement reference counting while
        casting objects in a class hierarchy in the <module|extend> module.
      </rsection>

      <\rsection|Module pair <module|concrete_null<format|enable indentation
      after>>>
        <code|>This module is used for concrete classes of objects that can
        meaningfully be unset. Unset object are represented with a
        <const|<value|NULL>> <var|rep> pointer, and are called <em|nil>
        references.

        <expand|paragraph|Declaration module>

        <\code>
          <cpp|#module> <module|concrete_null>(<var|T>, <var|R>)

          <indent|<type|R<apply|oper|*>> <var|rep>;<expand|//|memory
          representation>>

          <keyw|public>:

          <\indent>
            <keyw|inline> <funcproto||T|>;<expand|//|create a nil reference>

            <keyw|inline> <funcproto||T|<param|<keyw|const>
            T<apply|oper|&>|x>>;<expand|//|increase the reference count>

            <keyw|inline> <funcproto||<apply|oper|~>T|>;<expand|//|decrease
            the reference count>

            <keyw|inline> <operproto|T<apply|oper|*>||>;<expand|//|access to
            data members>

            <keyw|inline> <operproto|T|=|<param|T|x>>;<expand|//|increase the
            reference count>

            <keyw|inline> <keyw|friend> <funcproto|bool|nil|<param|T|x>>;<exp\
            and|//|is the reference unset?>
          </indent>
        </code>

        This module:

        <\itemize>
          <item>redefines the reference counting semantics of
          <module|concrete> to avoid counting <const|NULL> references;

          <item>defines an additional parameterless constructor, which
          produces a nil reference;

          <item>defines a <funcproto|bool|nil|<param|T|>> observer method
          that says whether the reference is nil or not.
        </itemize>
      </rsection>

      <\rsection|Module pair <module|abstract_null>>
        This module pair implement the semantic union of <module|abstract>
        and <module|concrete_null> module pairs.

        <expand|paragraph|Declaration module>

        <\code>
          <cpp|#module> <module|abstract_null> (<var|T>, <var|R>)

          <cpp|#import> <module|concrete_null> (<var|T>, <var|R>)

          <indent|<keyw|inline> <funcproto||T|<param|R<apply|oper|*>|rep>>;<e\
          xpand|//|increase the reference count>>

          <cpp|#endmodule>
        </code>
      </rsection>

      <\rsection|Extending reference counting classes>
        <\code>
          <cpp|#module> <module|extend> (<var|B>, <var|T>, <var|R>)

          <cpp|#import> <module|abstract> (<var|T>, <var|R>)

          <\indent>
            <keyw|inline> <funcproto||T|>;<expand|//|count references inside
            class <type|T>>

            <keyw|inline> <funcproto||T|<param|B|x>>;<expand|//|cast to down
            to <type|T>, increase reference count>

            <keyw|inline> <castproto|B>;<expand|//|cast up from <type|T>,
            <em|conserve> reference count>
          </indent>

          <cpp|#endmodule>
        </code>

        <\code>
          <cpp|#module> <module|extend_null> (<var|B>, <var|T>,
          <var|R>)<expand|//|like <module|extend>, but do not count <NULL>
          references>

          <cpp|#import> <module|abstract_null> (<var|T>, <var|R>)

          <\indent>
            <keyw|inline> <funcproto||T|<param|<var|B>|x>>;

            <keyw|inline> <castproto|B>;
          </indent>

          <cpp|#endmodule>
        </code>

        \ The two other module pairs, <module|extend> and
        <module|extend_null>, provide reference counting wich is robust to
        typecasting inside a class hierarchy. Multiple inheritance is not
        currently supported. The extension modules should be imported in
        specialization classes (children), and the root class should import
        the <module|abstract> or <module|abstract_null> module.

        \ The extension modules:

        <\itemize>
          <item>Define a cast operator returning a class <type|B> object
          constructed by copy from the <var|rep> member. This defines casting
          up to a parent class. The casted out reference is <em|not counted>
          by the class <type|T>, but since <type|B> is an abstract class, it
          must import <module|abstract> or <module|abstract_null>, and its
          constructor will increase the reference count.

          <item>Import the corresponding module pair for abstract classes.

          <item>Make reference counting transparent through a cast from
          another reference counting class <type|B> to the class <type|T>.
          This allows an object to be cast from a parent class down to a
          child class.<format|next line><strong|Caution.> This specializing
          cast is done using C-style pointer typecasting, which is
          <em|unsafe>.
        </itemize>

        \ If this seems obscure, just look at the code, it is deadly simple.

        These modules each take three parameters:

        <\description>
          <expand|item*|<type|B>>Name of the parent (generalization) class of
          <type|T>.

          <expand|item*|<var|T>>Name of the class in which the module is
          imported (specialization).

          <expand|item*|<var|R>>Name of the memory representation class of
          <type|T>.
        </description>

        <\remark>
          \ Multiple inheritance support could be implemented by defining an
          <module|implement> (resp. <module|implement_null>) module pair
          which would not import the <module|abstract> (resp.
          <module|abstract_null>) module pair, but would otherwise be
          identical to <module|extend> (resp. <module|extend_null>). The
          choice of names suggests that multiple inheritance would be used
          only to share interfaces, not implementation. Actually, I do not
          think that anyone sane can use multiple inheritance to share
          implementation and still claim its code to be maintainable.
        </remark>
      </rsection>
    </rsection>
  </rsection>

  <\rsection|Modules>
    <\rsection|Module pair <module|array>(<type|T>)>
      Declares and defines an explicit instanciation of the class template
      <template|array|<param|T>>.
    </rsection>

    <rsection|Module pair <module|string>|Declares and defines the class
    <type|string>.>

    <rsection|Module pair <module|tree>|Declares and defines the class
    <type|tree>.>
  </rsection>

  <\rsection|Classes>
    <\class|<template|array|<param|T>>>
      Fixed size arrays with reference counting and pointer copying.

      Imports module pair <module|concrete> (<template|array|T>,
      <template|array_rep|<var|T>>).

      Let <with|mode|math|N> be the logical size of the array, and
      <with|mode|math|\<Phi\>> its physical size,

      <expand|equation*|<left|{><expand|tabular*|<tformat|<table|<row|<cell|\\
      <Phi\>=N>|<cell|<with|mode|text|if >N\<leqslant\>5>>|<row|<cell|\<Phi\>\
      =2<rsup|p><mid|\\>p = min<left|(>x<mid|\\>2<rsup|x>\<geqslant\>N<right|\
      )>>|<cell|otherwise>>>>>>

      Thus, resizing operations are kept to a minimum. Moreover, since
      <with|font shape|small-caps|GNU> <expand|func|malloc>() reserves memory
      blocks whose sizes are powers of two, no memory is wasted if
      <keyw|sizeof>(<type|T>) is a power of two.

      The string representation of an empty array is <litstr|[]>.

      The string reprensentation of a non empty array is
      <litstr|[<with|mode|math|<left|langle>item<right|rangle>>,<with|mode|ma\
      th|\<cdots\> <left|langle>item<right|rangle>>]>.

      <\constructors>
        <ctorproto|<optparam|int|n|0>><expand|//|Uninitialized array of size
        <var|n>>

        <ctorproto|<param|T<apply|oper|*>|a><param|int|n>><expand|//|Initiali\
        ze from a <value|c>-style array of size <var|n>>
      </constructors>

      <\members>
        <operproto|T<apply|oper|&>|[]|<param|int|i>><expand|//|Reference to
        element <var|i>>

        <castproto|tree><expand|//|Cast to a tree labeled <const|TUPLE> whose
        subtrees are <type|T>s cast to <type|tree>>
      </members>

      <\functions>
        <funcproto|int|N|<param|<value|theclass>|a>><expand|//|Logical size
        of the array>

        <funcproto|<value|theclass>|copy|<param|<value|theclass>|a>><expand|/\
        /|Independant copy or the array>

        <operproto|bool|==|<param|<value|theclass>|a><param|<value|theclass>|\
        b>><expand|//|<with|mode|math|N(a)=N(b)> and all their elements are
        equal>

        <operproto|bool|!=|<param|<value|theclass>|a><param|<value|theclass>|\
        b>><expand|//|<with|mode|math|N(a)\<neq\>N(b)> or any of their
        elements differs>

        <operproto|ostream<apply|oper|&>|\<less\>\<less\>|<param|ostream<appl\
        y|oper|&>|out><param|<value|theclass>|a>><expand|//|Print the string
        representation>

        <operproto|<value|theclass><apply|oper|&>|\<less\>\<less\>|<param|<va\
        lue|theclass><apply|oper|&>|a><param|T|x>><expand|//|Append the
        element <var|x> to the array <var|a>>

        <operproto|<value|theclass><apply|oper|&>|\<less\>\<less\>|<param|<va\
        lue|theclass><apply|oper|&>|a><param|<value|theclass>|b>><expand|//|A\
        ppend the array <var|b> to the array <var|a>>
      </functions>
    </class>

    <\class|string>
      Fixed size strings with reference counting and pointer copying. Null
      characters are allowed in <type|string>s.

      Imports module pair <module|concrete> (<type|string>,
      <type|string_rep>).

      The string representation of a <type|string> is the sequence of all the
      characters of the <type|string> including null characters. This string
      representation contains no delimiters or special characters.

      Let <mode|math|N> be the logical size of the string, and
      <mode|math|\<Phi\>> its physical size,

      <expand|equation*|<left|{><expand|tabular*|<tformat|<table|<row|<cell|\\
      <Phi\>=N<with|mode|text|>>|<cell|<with|mode|text|if
      >>|<cell|N\<leqslant\>20>>|<row|<cell|\<Phi\>=32>|<cell|<with|mode|text\
      |if >>|<cell|21\<leqslant\>N\<leqslant\>32>>|<row|<cell|\<Phi\>=2<rsup|\
      p><mid|\\>p = min<left|(>x<mid|\\>2<rsup|x>\<geqslant\>N<right|)>>|<cel\
      l|>|<cell|otherwise>>>>>>

      <\constructors>
        <ctorproto|><expand|//|Empty string>

        <ctorproto|<param|int|n>><expand|//|String of <var|n> random
        characters>

        <ctorproto|<param|char|c>><expand|//|String of the single character
        <var|c>>

        <ctorproto|<param|char<apply|oper|*>|s>><expand|//|Initialize from a
        <value|c>-style string>

        <ctorproto|<param|char<apply|oper|*>|s><param|int|n>><expand|//|initi\
        alize string from a buffer>
      </constructors>

      <\members>
        <operproto|char<apply|oper|&>|[]|<param|int|i>><expand|//|Reference
        to element <var|i>>

        <operproto|bool|==|<param|char<apply|oper|*>|s>><expand|//|Equality
        to a <value|c>-style string>

        <operproto|bool|!=|<param|char<apply|oper|*>|s>><expand|//|Difference
        from a <value|c>-style string>

        <operproto|bool|==|<param|string|s>><expand|//|Equality to another
        <type|string>>

        <operproto|bool|!=|<param|string|s>><expand|//|Difference from
        another string>

        <operproto|string|()|<param|int|start><param|int|end>><expand|//|Subs\
        tring from index <var|start> to index
        <with|mode|math|<with|mode|text|<var|end>>-1>>
      </members>

      <\functions>
        <funcproto|int|N|<param|string|a>><expand|//|Logical size of the
        string>

        <funcproto|string|copy|<param|string|a>><expand|//|Independant copy
        of the string>

        <operproto|ostream<apply|oper|&>|\<less\>\<less\>|<param|ostream<appl\
        y|oper|&>|out><param|string|a>><expand|//|Write the string to a
        stream>

        <operproto|string<apply|oper|&>|\<less\>\<less\>|<param|string<apply|\
        oper|&>|a><param|char|c>><expand|//|Append the character <var|c> to
        <var|a>>

        <operproto|string<apply|oper|&>|\<less\>\<less\>|<param|string<apply|\
        oper|&>|a><param|string|b>><expand|//|Append the string <var|b> to
        <var|a>>

        <operproto|string|\<less\>\<less\>|<param|string|a><param|string|b>><\
        expand|//|New string, concatenation of <var|a> and <var|b>>

        <operproto|bool|\<less\>=|<param|string|a><param|string|b>><expand|//\
        |Lexcical comparison on raw character codes>

        <funcproto|int|hash|<param|string|s>><expand|//|Fast string hashing>

        <expand|paragraph|Conversion functions>

        <funcproto|bool|as_bool|<param|string|s>><expand|//|<litstr|true> is
        <const|true>, anything else is <const|false>>

        <funcproto|int|as_int|<param|string|s>><expand|//|Convert head of
        <var|s> matching <with|mode|math|<with|mode|text|-><rsup|?>[<with|mod\
        e|text|0-9>]<rsup|\<ast\>>>, the empty string gives
        <with|mode|math|0>>

        <funcproto|double|as_double|<param|string|s>><expand|//|Convert
        <var|s> to <type|double> using <expand|func|sscanf>>

        <funcproto|char<apply|oper|*>|as_charp|<param|string|s>><expand|//|Co\
        py <var|s> to a newly allocated <type|char<apply|oper|[]>> and append
        <litchr|\\0> to it>

        <funcproto|string|as_string|<param|int|i>><expand|//|New string
        containing <var|i> converted to a string using <expand|func|sprintf>>

        <funcproto|string|as_string|<param|double|x>><expand|//|<with|font
        shape|italic|Ibidem>>

        <funcproto|string|as_string|<param|char<apply|oper|*>|s>><expand|//|S\
        ynonym for the constructor <ctorproto|<param|char<apply|oper|*>|s>>>

        <funcproto|string|unquote|<param|string|s>><expand|//|Remove first
        and last characters if and only if they are both <litchr|">>

        <expand|paragraph|Predicate functions>

        <funcproto|bool|is_bool|<param|string|s>><expand|//|<var|s> is
        <litstr|true> or <litstr|false>>

        <funcproto|bool|is_int|<param|string|s>><expand|//|<var|s> matches
        <verbatim|<with|mode|math|<with|mode|text|+><rsup|?><with|mode|text|-\
        ><rsup|?>[<with|mode|text|0-9>]<rsup|+>>>>

        <funcproto|bool|is_double|<param|string|s>><expand|//|<var|s> matches
        <verbatim|<with|mode|math|<with|mode|text|+><rsup|?><with|mode|text|-\
        ><rsup|?>[<with|mode|text|0-9>]<rsup|\<ast\>>(\\<with|mode|text|.>[<w\
        ith|mode|text|0-9>]<rsup|+>)<rsup|?>(<with|mode|text|e><with|mode|tex\
        t|+><rsup|?><with|mode|text|-><rsup|?>[<with|mode|text|0-9>]<rsup|+>)\
        <rsup|?>>>>

        <funcproto|bool|is_charp|<param|string|s>><expand|//|Always true (may
        be usefule in templates)>

        <funcproto|bool|is_quoted|<param|string|s>><expand|//|<var|s> matches
        <verbatim|<with|mode|math|<with|mode|text|".><rsup|\<ast\>><with|mode\
        |text|">>>>

        <funcproto|bool|is_id|<param|string|s>><expand|//|<var|s> matches
        <verbatim|<with|mode|math|[<with|mode|text|a-zA-Z_>][<with|mode|text|\
        a-zA-Z_0-9>]<rsup|\<ast\>>>>>
      </functions>
    </class>

    <vspace*|1fn>One could notice that the patterns matched by
    <expand|func|as_int> and <expand|func|to_int> are not the same. This
    seems to be a bug.

    <with|color|red|<with|font series|bold|Todo: >Error messages>

    <\class|tree>
      Fixed size trees with reference counting.

      Imports module pair <module|concrete> (<type|tree>, <type|tree_rep>).

      <\itemize>
        <item>Nodes and leaves of tree are labeled by <type|string>s.

        <item>The definition of tree is recursive: a tree is a label (root
        node) and a serie of zero or more subtrees.

        <item>The <em|label of the tree> is a shorthand for the <em|label of
        the root node of the tree>.

        <item>A leaf is a tree with exactly zero subtree.

        <item>An empty leaf is a leaf labeled by the empty <type|string>.
      </itemize>

      The string representation of leaf is
      ``<with|mode|math|\<langle\>>label<with|mode|math|\<rangle\>>''.

      The string representation of a tree is
      ``<with|mode|math|<left|langle>>label<with|mode|math|<right|rangle>>
      <with|font family|tt|(><with|mode|math|<left|langle>subtree<right|rangl\
      e>>, <with|mode|math|\<cdots\>> <with|mode|math|<left|langle>subtree<ri\
      ght|rangle>><with|font family|tt|)>''.

      <with|color|red|<with|font series|bold|Warning:> This is very likely to
      have changed in later releases>

      <\constructors>
        <ctorproto|><expand|//|Empty tree: one-node tree labeled by the empty
        string>

        <ctorproto|<param|string|l><optparam|int|n|0>><expand|//|Tree labeled
        with <var|l> , with <with|mode|math|n> empty subtrees>

        <vspace*|0.5fn>The following constructors make a tree out of a label
        and a number of subtrees.

        <ctorproto|<param|string|l><param|tree|t1>>

        <ctorproto|<param|string|l><param|tree|t1><param|tree|t2>>

        <ctorproto|<param|string|l><param|tree|t1><param|tree|t2><param|tree|\
        t3>>

        <ctorproto|<param|string|l><param|tree|t1><param|tree|t2><param|tree|\
        t3><param|tree|t4>>
      </constructors>

      <\members>
        <operproto|tree<apply|oper|&>|[]|<param|int|i>><expand|//|Reference
        to the <with|mode|math|(i+1)>th subtree>

        <funcproto|void|resize|<param|int|n2>><expand|//|Set the number of
        subtrees to <var|n2>>

        <operproto|tree|()|<param|int|start><param|int|end>><expand|//|Tree
        with the same label and subtrees <var|start> to
        <with|mode|math|(<with|mode|text|<var|end>>-1)>>
      </members>

      <\functions>
        <funcproto|tree|copy|<param|tree|t>><expand|//|New memory
        reprensentation of <var|t>>

        <funcproto|int|N|<param|tree|t>><expand|//|Number of children of the
        root node>

        <operproto|bool|==|<param|tree|t><param|tree|u>>

        <operproto|bool|!=|<param|tree|t><param|tree|u>>

        <operproto|tree|\<less\>\<less\>|<param|tree<apply|oper|&>|t><param|t\
        ree|t2>><expand|//|Append <var|t2> to the array of subtrees of
        <var|t>>

        <operproto|tree<apply|oper|&>|\<less\>\<less\>|<param|tree<apply|oper\
        |&>|t><param|<template|array|<param|tree>>|a>><expand|//|Append
        <var|a> to the array of subtrees of <var|t>>

        <operproto|ostream<apply|oper|&>|\<less\>\<less\>|<param|ostream<appl\
        y|oper|&>|out><param|tree|t>><expand|//|Print the string
        representation>

        <vspace*|0.5fn>A <type|tree> can be considered a function. In this
        case, the function name is the label of the tree and the function
        parameters are the subtrees. The following predicates test whether a
        tree is a function of a given name and given arity.

        <with|font series|bold|Note: >These are the only functions which test
        the label of a tree.<vspace|0.5fn>

        <funcproto|bool|is_func|<param|tree|t><param|string|s>><expand|//|Has
        <var|t> at least one subtree and the label <var|s>?>

        <funcproto|bool|is_func|<param|tree|t><param|char<apply|oper|*>|s>><e\
        xpand|//|Same, but test the label against a C-style string>

        <funcproto|bool|is_func|<param|tree|t><param|string|s><param|int|i>><\
        expand|//|Has <var|t> exactly <var|n> subtrees and the label
        <var|s>?>

        <funcproto|bool|is_func|<param|tree|t><param|char<apply|oper|*>|s><pa\
        ram|int|i>><expand|//|Same, but test the label against a C-style
        string>
      </functions>
    </class>

    <\class>
      <template|list|<param|T|>>
    <|class>
      Linked lists with reference counting. The <template|list_rep|<param|T>>
      constructor, which creates new cells, allows cells to be shared across
      multiple lists. This semantic is identical to the list semantic in
      Lisp: lists can share a tail, and modifying one's tail would change the
      other's tail.

      List operations are implemented recursively but are not tail recursive.
      This could be considered a performance bug, and may result in stack
      overflows.

      Imports module pair <module|concrete_null>(<type|<theclass>>,
      <template|list_rep|<param|T>>).

      The string representation of an empty list is ``<with|font
      family|tt|[]>''.

      The string reprensentation of a non empy list is ``<with|font
      family|tt|[><with|mode|math|<left|langle>item<right|rangle>>,
      <with|mode|math|\<cdots\>> <with|mode|math|<left|langle>item<right|rang\
      le>><with|font family|tt|]>''.

      <\constructors>
        <ctorproto|><expand|//|Empty list>

        <ctorproto|<param|T|item>><expand|//|New list containing only
        <var|item>>

        <ctorproto|<param|T|item><param|<theclass>|next>><expand|//|New list
        made from an <var|item> and a tail (<var|next>)>
      </constructors>

      <\members>
        <operproto|T&|[]|<param|int|i>><expand|//|Reference to
        <with|mode|math|(<with|mode|text|<var|i>>+1)<rsup|<with|mode|text|th>\
        >> item of the list>

        <castproto|tree><expand|//|Cast to a tree labeled <const|TUPLE> whose
        subtrees are <type|T>s casted to <type|tree>>
      </members>

      <\functions>
        <funcproto|int|N|<param|<apply|theclass>|l>><expand|//|Number of
        items in list <var|l>>

        <funcproto|<apply|theclass>|copy|<param|<theclass>|l>><expand|//|New
        memory representation of <var|l>>

        <funcproto|<value|theclass>|merge|<param|<value|theclass>|l1><param|<\
        value|theclass>|l2>><expand|//|Concatenation of <var|l1> and
        <var|l2>. Tail is <em|shared>>

        <funcproto|<value|theclass>|head|<param|<value|theclass>|l><optparam|\
        int|n|1>><expand|//|New list made of the first <var|n> items of
        <var|l>>

        <funcproto|<value|theclass>|tail|<param|<value|theclass>|l><optparam|\
        int|n|1>><expand|//|Last <var|n> items of <var|l>, shared with
        <var|l>>

        <funcproto|<value|theclass>|revert|<param|<value|theclass>|l>><expand\
        |//|New list which has items in reverse order of <var|l>>

        <operproto|ostream&|\<less\>\<less\>|<param|<value|theclass>|l>><expa\
        nd|//|Write the string representation to a stream>

        <expand|paragraph|Insertion and suppression functions>

        <operproto|<value|theclass>&|\<less\>\<less\>|<param|<value|theclass>\
        &|l><param|T|item>><expand|//|Append <var|item> to <var|l>>

        <operproto|<value|theclass>&|\<less\>\<less\>|<param|<value|theclass>\
        &|l1><param|<value|theclass>&|l2>><expand|//|Make <var|l2> the tail
        of <var|l1> (append <var|l2> to <var|l1>)>

        <operproto|<value|theclass>&|\<gtr\>\<gtr\>|<param|T|item><param|<val\
        ue|theclass>&|l>><expand|//|Set <var|l> to a new <theclass> whose
        head is <var|item> and tail is <var|l>>

        <operproto|<value|theclass>&|\<less\>\<less\>|<param|T|item><param|<v\
        alue|theclass>&|l>><expand|//|Pop the head off <var|l> and into
        <var|item>>

        <funcproto|<value|theclass>&|suppress_last|<param|<value|theclass>&|l\
        >><expand|//|Drop the last item off <var|l> (and lists sharing its
        tail)>

        <funcproto|T&|last_item|<param|<value|theclass>|l>><expand|//|Referen\
        ce to <var|item> of last cell of <var|l>>

        <expand|paragraph|Predicate functions>

        <funcproto|bool|atom|<param|<value|theclass>|l>><expand|//|Is <var|l>
        a single element list?>

        <operproto|bool|==|<param|<value|theclass>|l1><param|<value|theclass>\
        |l2>><expand|//|<with|mode|math|N(<with|mode|text|<var|l1>>)\<neq\>N(\
        <with|mode|text|<var|l2>>)> or any of their elements differs>

        <operproto|bool|\<less\>|<param|<value|theclass>|l1><param|<value|the\
        class>|l2>><expand|//|<var|l1> is equal to head of <var|l2> and
        <with|mode|math|N(<with|mode|text|<var|l1>>)\<less\>N(<with|mode|text\
        |<var|l2>>)>>

        <operproto|bool|\<less\>=|<param|<value|theclass>|l1><param|<value|th\
        eclass>|l2>><expand|//|<var|l1> is equal to head of <var|l2> and
        <with|mode|math|N(<with|mode|text|<var|l1>>)\<leqslant\>N(<with|mode|\
        text|<var|l2>>)>>
      </functions>
    </class>

    <subsection|Type <type|path>>

    Paths are integer lists, wich are for instance useful to select subtrees
    in trees.

    <keyw|typedef> <template|list|<param|int|>> <type|path>
  </rsection>
</body>

<\initial>
  <\collection>
    <associate|preamble|false>
    <associate|page right margin|30mm>
    <associate|reduction page left margin|27mm>
    <associate|page width|336384unit>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page medium|papyrus>
    <associate|shrinking factor|5>
    <associate|page top margin|30mm>
    <associate|font|bookman>
    <associate|reduction page right margin|27mm>
    <associate|paragraph hyphenation|normal>
    <associate|reduction page bottom margin|27mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|page height|415744unit>
    <associate|reduction page top margin|27mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-40|<tuple|5.15|?>>
    <associate|toc-30|<tuple|5.7|?>>
    <associate|toc-10|<tuple|3.2.2|?>>
    <associate|toc-20|<tuple|4.1|?>>
    <associate|toc-41|<tuple|5.16|?>>
    <associate|toc-21|<tuple|4.2|?>>
    <associate|toc-11|<tuple|3.2.2|?>>
    <associate|toc-31|<tuple|5.8|?>>
    <associate|toc-42|<tuple|5.16|?>>
    <associate|toc-22|<tuple|4.3|?>>
    <associate|toc-12|<tuple|3.2.3|?>>
    <associate|toc-32|<tuple|5.8|?>>
    <associate|toc-43|<tuple|5.16|?>>
    <associate|toc-33|<tuple|5.8|?>>
    <associate|toc-13|<tuple|3.2.3|?>>
    <associate|toc-23|<tuple|5|?>>
    <associate|toc-44|<tuple|5.17|?>>
    <associate|toc-34|<tuple|5.9|?>>
    <associate|toc-14|<tuple|3.2.4|?>>
    <associate|toc-24|<tuple|5.1|?>>
    <associate|toc-45|<tuple|22.1|?>>
    <associate|toc-35|<tuple|5.10|?>>
    <associate|toc-15|<tuple|3.2.4|?>>
    <associate|toc-25|<tuple|5.2|?>>
    <associate|toc-46|<tuple|22|?>>
    <associate|toc-36|<tuple|5.11|?>>
    <associate|toc-16|<tuple|3.2.5|?>>
    <associate|toc-26|<tuple|5.3|?>>
    <associate|toc-47|<tuple|22|?>>
    <associate|toc-37|<tuple|5.12|?>>
    <associate|toc-27|<tuple|5.4|?>>
    <associate|toc-17|<tuple|3.2.5|?>>
    <associate|toc-48|<tuple|22.1|?>>
    <associate|toc-38|<tuple|5.13|?>>
    <associate|toc-28|<tuple|5.5|?>>
    <associate|toc-18|<tuple|3.2.6|?>>
    <associate|toc-39|<tuple|5.14|?>>
    <associate|toc-29|<tuple|5.6|?>>
    <associate|toc-19|<tuple|4|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|what is a module?|<tuple|2.1|?>>
    <associate|toc-2|<tuple|2|?>>
    <associate|toc-3|<tuple|2.1|?>>
    <associate|toc-4|<tuple|2.2|?>>
    <associate|toc-5|<tuple|2.3|?>>
    <associate|toc-6|<tuple|3|?>>
    <associate|toc-7|<tuple|3.1|?>>
    <associate|toc-8|<tuple|3.2|?>>
    <associate|toc-9|<tuple|3.2.1|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font series|<quote|bold>|1<space|2spc>About this
      document><value|toc-dots><pageref|toc-1><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|2<space|2spc>Conventions><v\
      alue|toc-dots><pageref|toc-2><vspace|0.5fn>

      2.1<space|2spc>Parameters <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      shape|<quote|italic>|<with|mode|<quote|math>|<left|langle><with|mode|<q\
      uote|text>|name<with|mode|<quote|math>|<right|rangle>>>>2>><value|toc-d\
      ots><pageref|toc-3>

      2.2<space|2spc>Modules<value|toc-dots><pageref|toc-4>

      2.3<space|2spc>Module conventions<value|toc-dots><pageref|toc-5>

      <vspace*|1fn><with|font series|<quote|bold>|3<space|2spc>Reference
      counting><value|toc-dots><pageref|toc-6><vspace|0.5fn>

      3.1<space|2spc>What is reference counting<value|toc-dots><pageref|toc-7\
      >

      3.2<space|2spc>Reference counting in
      T<rsub|<space|-0.4spc><move|<resize|<with|index
      level|<quote|0>|E>||||0.5fn|>|0fn|-0.1fn>><space|-0.4spc>X<rsub|<space|\
      -0.4spc><move|<resize|M<space|-0.2spc>A<space|-0.4spc>CS||||0.5fn|>|0fn\
      |-0.1fn>><value|toc-dots><pageref|toc-8>

      <with|left margin|<quote|1.5fn>|3.2.1<space|2spc>Classes
      conventions<value|toc-dots><pageref|toc-9>>

      <with|left margin|<quote|1.5fn>|3.2.2<space|2spc>Module pair <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|tt>|font shape|<quote|italic>|concrete>><value|toc-dots><\
      pageref|toc-10>>

      <with|left margin|<quote|1.5fn>|3.2.3<space|2spc><value|toc-dots><pager\
      ef|toc-11>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Declaration
      module<value|toc-dots><pageref|toc-12>>

      <with|left margin|<quote|1.5fn>|3.2.4<space|2spc>Module pair <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|tt>|font shape|<quote|italic>|abstract>><value|toc-dots><\
      pageref|toc-13>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Declaration
      module<value|toc-dots><pageref|toc-14>>

      <with|left margin|<quote|1.5fn>|3.2.5<space|2spc>Module pair <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|tt>|font shape|<quote|italic>|concrete_null<format|enable
      indentation after>>><value|toc-dots><pageref|toc-15>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Declaration
      module<value|toc-dots><pageref|toc-16>>

      <with|left margin|<quote|1.5fn>|3.2.6<space|2spc>Module pair <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|tt>|font shape|<quote|italic>|abstract_null>><value|toc-d\
      ots><pageref|toc-17>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Declaration
      module<value|toc-dots><pageref|toc-18>>

      <with|left margin|<quote|1.5fn>|3.2.7<space|2spc>Extending reference
      counting classes<value|toc-dots><pageref|toc-19>>

      <vspace*|1fn><with|font series|<quote|bold>|4<space|2spc>Modules><value\
      |toc-dots><pageref|toc-20><vspace|0.5fn>

      4.1<space|2spc>Module pair <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|tt>|font
      shape|<quote|italic>|array>>(<with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|T>>)<value|toc-dots><p\
      ageref|toc-21>

      4.2<space|2spc>Module pair <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|tt>|font
      shape|<quote|italic>|string>><value|toc-dots><pageref|toc-22>

      4.3<space|2spc>Module pair <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|tt>|font
      shape|<quote|italic>|tree>><value|toc-dots><pageref|toc-23>

      <vspace*|1fn><with|font series|<quote|bold>|5<space|2spc>Classes><value\
      |toc-dots><pageref|toc-24><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|6<space|2spc>Class
      <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|<with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|rm>|font series|<quote|medium>|font
      shape|<quote|slanted>|array>><with|mode|<quote|math>|<left|langle>><wit\
      h|param-sep|<quote|>|param|<quote|<macro|t|n|<value|param-sep><type|<ar\
      g|t>><assign|param-sep|, >>>|<with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|T>><assign|param-sep|<\
      quote|, >>><with|mode|<quote|math>|<right|rangle>>>>><value|toc-dots><p\
      ageref|toc-25><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|7<space|2spc>Constructors><\
      value|toc-dots><pageref|toc-26><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|8<space|2spc>Members><value\
      |toc-dots><pageref|toc-27><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|9<space|2spc>Functions><val\
      ue|toc-dots><pageref|toc-28><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|10<space|2spc>Class
      <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|string>>><value|toc-do\
      ts><pageref|toc-29><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|11<space|2spc>Constructors>\
      <value|toc-dots><pageref|toc-30><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|12<space|2spc>Members><valu\
      e|toc-dots><pageref|toc-31><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|13<space|2spc>Functions><va\
      lue|toc-dots><pageref|toc-32><vspace|0.5fn>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Conversion
      functions<value|toc-dots><pageref|toc-33>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Predicate
      functions<value|toc-dots><pageref|toc-34>>

      <vspace*|1fn><with|font series|<quote|bold>|14<space|2spc>Class
      <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|tree>>><value|toc-dots\
      ><pageref|toc-35><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|15<space|2spc>Constructors>\
      <value|toc-dots><pageref|toc-36><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|16<space|2spc>Members><valu\
      e|toc-dots><pageref|toc-37><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|17<space|2spc>Functions><va\
      lue|toc-dots><pageref|toc-38><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|18<space|2spc>Class
      <with|font series|<quote|medium>|math font
      series|<quote|bold>|<\with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>>
        <with|font series|<quote|medium>|math font
        series|<quote|bold>|<with|font family|<quote|rm>|font
        series|<quote|medium>|font shape|<quote|slanted>|list>><with|mode|<qu\
        ote|math>|<left|langle>><with|param-sep|<quote|>|param|<quote|<macro|\
        t|n|<value|param-sep><type|<arg|t>><assign|param-sep|, >>>|<with|font
        series|<quote|medium>|math font series|<quote|bold>|<with|font
        family|<quote|rm>|font series|<quote|medium>|font
        shape|<quote|slanted>|T>><assign|param-sep|<quote|,
        >>><with|mode|<quote|math>|<right|rangle>>
      </with>>><value|toc-dots><pageref|toc-39><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|19<space|2spc>Constructors>\
      <value|toc-dots><pageref|toc-40><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|20<space|2spc>Members><valu\
      e|toc-dots><pageref|toc-41><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|21<space|2spc>Functions><va\
      lue|toc-dots><pageref|toc-42><vspace|0.5fn>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Insertion and
      suppression functions<value|toc-dots><pageref|toc-43>>

      <with|left margin|<quote|6fn>|font size|<quote|0.84>|Predicate
      functions<value|toc-dots><pageref|toc-44>>

      21.1<space|2spc>Type <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|path>><value|toc-dots>\
      <pageref|toc-45>
    </associate>
  </collection>
</auxiliary>
