<TeXmacs|0.3.5.14>

<style|<tuple|article|allouche>>

<\body>
  <\expand|make-title>
    <title|Stepping inside <TeXmacs> 0.3.4.4>

    <author|David Allouche>

    <expand|title-date|Around may >2001
  </expand>

  <\rsection|What this is>
    This document is a description of the execution of <TeXmacs> 0.3.4.4
    (<with|font family|tt|http://www.texmacs.org>) as seen from <sh|gdb> (the
    notorious <with|font shape|small-caps|Gnu> DeBugger).

    This description is not exhaustive. It is done with a focus on the
    windowing system of <TeXmacs>, and how it could be adapted to GTK+.

    <\rsection|How to compile \ and install <TeXmacs> for debugging>
      These step by step instructions will describe how to compile install
      and run a modified <TeXmacs> system that will allow easy step by step
      execution using <sh|emacs>, or any other IDE.

      First you will need to modify the file <file|Makefile.in> at the base
      of the source tree. Find the line

      <with|font family|tt|STRIP=strip>

      and replace it with the line

      <with|font family|tt|STRIP=true>.

      This modification will prevent the <bash|make install> command from
      striping debugging information from the installed binary files.

      <remark|<sh|true> is a standard shell command that does nothing and
      exit with success. It is intended to be used in conditional constructs
      (whence the name). Here it is used as a no-op command.>

      Then you must modify the file <file|misc/scripts/texmacs.in>. This file
      is used to generate the <sh|texmacs> script that is normally run when
      one type <sh|texmacs> on the command line. This script sets some
      environment variables and run the actual binary program
      <sh|texmacs.bin>.

      Find the last line of <file|texmacs.in>

      <bash|texmacs.bin $*>

      and replace it by these lines (adapt this to you needs if you do not
      use <sh|emacs>)

      <noparskip|<\bash>
        if [ `basename $0` = 'texmacs-dev' ]

        then echo "Starting emacs in texmacs environment" ; emacs

        else texmacs.bin $*

        fi
      </bash>>

      <vspace*|0.5fn>Now, run the <sh|configure> script. You should specify a
      special <shopt|---prefix> option to avoid replacing your production
      <TeXmacs> with the debugger enabled one; for example
      <shopt|---prefix=/usr/local/TeXmacs-dev>. The <sh|configure> script
      will generate <file|Makefile> and <file|texmacs> customized with the
      changes we made in the <file|*.in> files.

      Then, set the <with|font family|tt|CXXFLAGS> environment variable, and
      then run <sh|make> with the <shopt|-e> option. This option tells
      <sh|make> to override any variable defined in the Makefile with defined
      environment variables of the same name.

      <bash|export CXXFLAGS=-ggdb; make -e && make install>

      Finally, go some directory you use to keep links to locally installed
      software (<dir|/usr/local/bin> ?) and create a symbolic link to the
      debugging <sh|texmacs>, with the name <sh|texmacs-dev>.

      <bash|cd /usr/local/bin && ln -s ../TeXmacs-dev/bin/texmacs
      texmacs-dev>
    </rsection>

    <rsection|How to step into <TeXmacs>|Now you can run <sh|texmacs-dev>, it
    will set the environment up and will lauch <sh|emacs>, then you can start
    the debugger mode with <with|font family|tt|M-x<space|1spc>gdb> and open
    the <with|font family|tt|$PREFIX/libexec/TeXmacs-0.3.4.4/bin/texmacs.bin>\
    . Then, just use <sh|gdb>.>
  </rsection>

  <rsection|Stepping into it!|<\rsection|Initialization>
    <\rsection|initialize_data() in Basic/Data/tree.gen.cc>
      Initialize a set of internal structures. <var|CONSTRUCTOR_CODE> is a
      <type|hashmap> whose keys are strings naming the types of allowed tree
      in documents, and whose values are indices used in the three follwing
      structures:

      <\enumerate>
        <item><var|CONSTRUCTOR_NAME> is a vector of name of trees as string.
        Peeking <var|CONSTRUCTOR_NAME> by index is the inverse operation of
        peeking <var|CONSTRUCTOR_CODE> by name. The indices are defined in
        <file|Basic/Data/tree.gen.h>.

        <item><var|CONSTRUCTOR_ARITY> is a vector of <type|int>s. The arity
        of the constructor (whatever that means). It can be
        <with|mode|math|-1> (for variable length parameter list),
        <with|mode|math|0> for the types <with|font family|tt|UNKNOWN and
        UNINIT>, and 1 or more for normal fixed arity constructor.

        <item><var|CONSTRUCTOR_PROPERTIES> is a vector of <type|int>s. There
        a two properties:

        <\itemize>
          <item>Accessibility, which uses the three lower bits (get using the
          mask <const|ACCESSIBLE_MASK>) and is one of <const|NOT_ACCESSIBLE>,
          <const|ACCESSIBLE>, <const|FIRST_ACCESSIBLE>,
          <const|LAST_ACCESSIBLE>, <const|TAIL_ACCESSIBLE> or
          <const|TABLE_ACCESSIBLE>.

          <item>Dynamic flag, wich can be tested and set using the values
          <const|DYNAMIC> or <const|DYNAMIC_MASK>
        </itemize>
      </enumerate>

      All this data is obviously related to the creation of the document
      tree, but how?
    </rsection>

    <rsection|Environment setting by install_texmacs() in
    Basic/System/dir.gen.cc|Apart from calling <expand|func|initialize_data>,
    the <expand|func|install_texmacs> function does a lot of work regarding
    parsing environnement variables, creating the <with|font
    family|tt|~/.TeXmacs> directory, and setting environment variable. Not
    very exciting though.>

    <\rsection|Starting Guile and after>
      After <expand|func|install_texmacs>, the <expand|func|main> function
      calls <expand|func|install_guile> which starts the Guile environment
      and then calls <expand|func|TeXmacs_main> wich goes on the
      initialization.

      The first thing done is parsing the command line options. Nothing
      special.

      The second is calling the function <expand|func|install_tex>
      (<file|Basic/System/dir.gen.cc>) which will read or create the file
      <with|font family|tt|$TEXMACS_HOME_PATH/system/TEX_PATHS> and set a
      number static and global variables of <file|dir.gen.cc> that are used
      by <expand|func|load_tex>, which in turn is called by the constructors
      of <type|tex_font_rep> and <type|tex_rubber_font_rep>.
    </rsection>

    <\rsection|Initalizing the X display>
      Then the display is opened by <expand|func|open_display>. Opening the
      display is creating a new <type|x_display_rep>. This object constructor
      does several things:

      <\itemize>
        <item> opens a connection to the X server designated by the
        <with|font family|tt|$DISPLAY> environment variable (get a pointer to
        a X <with|font family|tt|Display> structure);

        <item>cache some information related to the display in member
        variables, screen, root window, geometry, colormaps;

        <item>create two <type|GC>s (graphic contexts) for the default
        screen, and save them in the instance variables <var|gc> and
        <var|pixmap_gc>;

        <item>save the command lines parameters and clear some misc.
        instances variables (<var|state>, <var|shadow>, <var|gswindow>,
        <var|ballonwin>);

        <item>set the graphics-exposure flag of the main <type|GC>
        (<var|this><apply|oper|-\<gtr\>><var|gc>);

        <item><qmeth|x_display_rep|get_xmodmap><format|next line>Set the
        value of the static variables <var|mod_shift>, <var|mod_ctrl>, and
        <var|mod_alt> to the list of keysyms used for the correponding
        modifiers. It seems that these static variables are never used after
        initialization. We tried to strip this code out and could not notice
        any

        <item><qmeth|x_display_rep|init_colormap><format|next line>Reserve a
        color map for 17 shades of grey and 125 colours spread uniformly
        around the color cube. The <var|cols> instance variable is the X
        color map used with X function, and the <var|cmap> is the list of the
        pixel values of the reserved colors. This part of the code is very
        strange, the author seems to have patched over the classical "count
        from 0 to <with|mode|math|n>" C mistake. Noticeably, black and white
        are allocated twice, and the intention seems to be 16 shades of grey
        and 64 colors...

        <item><qmeth|x_display_rep|initialize_colors><format|next line>After
        the colormap reservation, sets the colors instance variables
        inherited from the class <type|display_rep> to the pixel values of
        each colors (the variables names are <var|black>, <var|white>,
        <var|red>, etc.) using the <type|x_display_rep><apply|oper|>::<expand\
        |func|rgb> method.

        <item><qmeth|x_display_rep|initialize_keyboard_pointer><format|next
        line>Initialize the <var|lower_key> and <var|upper_key> instance
        variables. They are hash tables whose keys are keycodes and values
        are strings representing text associated to the key. They are single
        char string for ASCII letters, but for international letters they are
        sequences like <with|font family|tt|\\xe0>, for control and dead keys
        they are the name of the key (like <with|font family|tt|grave>,
        <with|font family|tt|space>, or <with|font family|tt|F12>) and two
        ASCII keys are associated to special sequence (\\ is associated to
        <with|font family|tt|\\\\>, and " is associated to <with|font
        family|tt|\\x22>). Common control key (whose caracter is not to be
        printed, but executed) like space, return, arrows or function keys
        are mapped differently (using the <expand|func|map> method, instead
        of the <expand|func|Map> method): the <var|upper_key> instance
        variable maps to <with|font family|tt|S-> followed by the name of the
        key, obviously that the name used in <TeXmacs> key sequence
        representations.

        <item>Initialize the <var|out_lan> instance variable (inherited from
        <type|display_rep>). This string is set to the name of the language
        (like <with|font family|tt|czech> or <with|font family|tt|french>)
        corresponding to the locale defined in the environement variables
        <with|font family|tt|LANG> or <with|font family|tt|GDM_LANG>. If none
        are set, it defaults to <with|font family|tt|english>.
      </itemize>
    </rsection>

    <\rsection|Initializing the <TeXmacs> server>
      Now that we have opened the display, we "start the server", that is we
      create a <type|server> object (and it internal <type|tm_server_rep>)
      with the just opened <type|display>. The <type|tm_server_rep>
      constructor does the following:

      <\itemize>
        <item>Initialize the instance variables <var|dis> (the X display),
        <var|vw> (current editor) and <var|banner_nr> (how far in the banner,
        whatever that means).

        <item>Sets the global (but only declared in <file|tm_server_gen.cc>)
        variable <var|the_server> to a <type|server> object pointing to
        <var|this>.

        <item>Call the <expand|func|initialize_guile> function (in
        <file|Guile/Scheme/evaluate.gen.cc>) that define a few Scheme
        functions whose name begin in <with|font family|tt|TeXmacs>, and
        initialize the Scheme variables <svar|buffer-menu> and
        <svar|project-buffer-menu> to empty lists, and then call
        <expand|func|initialize_glue> (<file|Guile/Glue/glue.gen.cc>).

        <item><expand|func|initialize_glue> seems to define a new Scheme data
        type (<em|smob>) <with|font family|tt|tree>, and then calls
        <expand|func|initialize_glue_*> routines for the Basic, Editor, and
        Server modules. These routines only task is to define a set of
        built-in procedures using the <expand|func|gh_new_procedure>
        function.

        <item>Back to the server constructor, we set the main event loop
        <em|interpose handler> to the function
        <expand|func|texmacs_interpose_handler>
        (<file|Texmacs/Server/tm_server.gen.cc>). All it does is send a
        <qmeth|server|interpose_handler> message to <with|font
        family|tt|><apply|oper|*><var|the_server>.

        <note|The class <type|server> ought to import the
        <module|abstract_null> module, and <var|the_server> ought not be a
        pointer.>

        <item>Similarly, <var|guile_eval_routine> is set to
        <expand|func|texmacs_guile_eval_routine>. This callback is only used
        by <expand|func|guile_eval> in <file|Basic/System/file.gen.c>, it
        parse the string given in parameter and sends the resulting
        <type|scheme_tree> to <apply|oper|*><var|the_server> with an
        <expand|func|exec_delayed> message.

        <item>Then the instance variable <var|out_lan> (inherited from
        <type|tm_config_rep>) is set to the value of the instance variable of
        the same name in the <type|display> parameter of the
        <type|tm_server_rep> constructor.

        <item>At this point, the init file is executed with the now
        initialized Scheme interpreter, specifically with the
        <expand|func|eval_scheme_any> function. Which in turn wrap the string
        in a call to the <sfunc|TeXmacs-guile-eval> Scheme function. This
        function just evaluate the string, convert the result back into a
        string and trigger the <sfunc|TeXmacs-guile-error-handler>.
      </itemize>

      Then, it's cool, the <TeXmacs> server has been initialized.
    </rsection>

    <\rsection|Loading the initial buffer>
      Back to the the <expand|func|TeXmacs_main> function, any file given at
      the command line is open using the <ptrmsg|server|load_generic_buffer>
      message. Optionally (depending on the <var|install_status> variable, a
      builtin startup buffer is loaded using <ptrmsg|server|load_buffer>.
      Eventually, if no buffer has still opened, a new <with|font
      family|tt|no name> buffer is created using <ptrmsg|server|open_window>.

      The <ptrmsg|server|open_window> message is actually
      <qmeth|tm_data_rep|open_window>. Since <type|tm_data_rep> is not an
      immediate base class of <type|<type|tm_server_rep>>, I think its time
      for a little inheritance tree...

      <remark|<type|tm_server_rep> is the concrete object that is
      instanciated in the <type|server>(<type|display>) constructor.>

      <expand|equation*|<tree|<with|mode|text|<type|tm_server_rep>>|<tree|<wi\
      th|mode|text|<type|tm_config_rep>>|<tree|<with|mode|text|<keyw|virtual>
      <type|server_rep>>|<with|mode|text|<type|abstract_struct>>>>|<tree|<wit\
      h|mode|text|<type|tm_project_rep>>|<tree|<with|mode|text|<type|tm_data_\
      rep>>|<tree|<with|mode|text|<keyw|virtual>
      <type|server_rep>>|\<cdots\>>>>|<tree|<with|mode|text|<type|tm_layout_r\
      ep>>|<tree|<with|mode|text|<keyw|virtual>
      <type|server_rep>>|\<cdots\>>>|<tree|<with|mode|text|<type|tm_scheme_re\
      p>>|<tree|<with|mode|text|<keyw|virtual>
      <type|server_rep>>|\<cdots\>>>>>

      <expand|func|open_window> basically just find a new unused name for the
      <with|font family|tt|no name> buffer and create a new buffer with this
      name and an empty document <type|tree> using the method
      <qmeth|tm_data_rep|new_buffer_in_new_window>.

      <center|<with|color|red|<with|font series|bold|<with|font size|2|Work
      in progress>>>>
    </rsection>

    <\rsection|The main event loop <qmeth|x_display_rep|event_loop> in
    <file|Window/X/x_loop.gen.c>>
      <\expand|enumerate-numeric>
        <item>The process is put to sleep for a fixed amount of time using
        <expand|func|select>. This amount of time can be as big as 1 second
        (<const|MAX_DELAY>) after two minutes (<const|SLEEP_AFTER>) of
        inactivity. Occurence of any event will not be observed before the
        end of the sleep, this account for the sometimes sluggish reaction of
        <TeXmacs> Then, event will be polled until the event queue is empty.
        Then the process will sleep for 1/100th of second and make screen
        updates. The sleep time will remain 1/100th of second until
        <const|MAX_DELAY> has passed.

        <item>The <em|interpose handler> is executed just after the wait and
        before window redrawing and alarm execution. Currently, this handler
        can only be \ <expand|func|texmacs_interpose_handler>
        (<file|Texmacs/Server/tm_server.gen.cc>).
      </expand>
    </rsection>
  </rsection>>
</body>

<\initial>
  <\collection>
    <associate|preamble|false>
    <associate|page right margin|30mm>
    <associate|reduction page left margin|25mm>
    <associate|page width|1376256unit>
    <associate|magnification|1>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page medium|papyrus>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|page type|a4>
    <associate|reduction page bottom margin|15mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|page height|1517824unit>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-10|<tuple|2.1.5|?>>
    <associate|toc-11|<tuple|2.1.6|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-12|<tuple|2.1.7|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-13|<tuple|2.1.8|?>>
    <associate|toc-3|<tuple|1.2|?>>
    <associate|toc-14|<tuple|2.1.9|?>>
    <associate|toc-4|<tuple|2|?>>
    <associate|toc-15|<tuple|3.1.7|?>>
    <associate|toc-5|<tuple|2.1|?>>
    <associate|toc-6|<tuple|2.1.1|?>>
    <associate|toc-7|<tuple|2.1.2|?>>
    <associate|toc-8|<tuple|2.1.3|?>>
    <associate|toc-9|<tuple|2.1.4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font series|<quote|bold>|1<space|2spc>What this
      is><value|toc-dots><pageref|toc-1><vspace|0.5fn>

      1.1<space|2spc>How to compile \ and install
      T<rsub|<space|-0.4spc><move|<resize|<with|index
      level|<quote|0>|E>||||0.5fn|>|0fn|-0.1fn>><space|-0.4spc>X<rsub|<space|\
      -0.4spc><move|<resize|M<space|-0.2spc>A<space|-0.4spc>CS||||0.5fn|>|0fn\
      |-0.1fn>> for debugging<value|toc-dots><pageref|toc-2>

      1.2<space|2spc>How to step into T<rsub|<space|-0.4spc><move|<resize|<wi\
      th|index level|<quote|0>|E>||||0.5fn|>|0fn|-0.1fn>><space|-0.4spc>X<rsu\
      b|<space|-0.4spc><move|<resize|M<space|-0.2spc>A<space|-0.4spc>CS||||0.\
      5fn|>|0fn|-0.1fn>><value|toc-dots><pageref|toc-3>

      <vspace*|1fn><with|font series|<quote|bold>|2<space|2spc>Stepping into
      it!><value|toc-dots><pageref|toc-4><vspace|0.5fn>

      2.1<space|2spc>Initialization<value|toc-dots><pageref|toc-5>

      <with|left margin|<quote|1.5fn>|2.1.1<space|2spc>initialize_data() in
      Basic/Data/tree.gen.cc<value|toc-dots><pageref|toc-6>>

      <with|left margin|<quote|1.5fn>|2.1.2<space|2spc>Environment setting by
      install_texmacs() in Basic/System/dir.gen.cc<value|toc-dots><pageref|to\
      c-7>>

      <with|left margin|<quote|1.5fn>|2.1.3<space|2spc>Starting Guile and
      after<value|toc-dots><pageref|toc-8>>

      <with|left margin|<quote|1.5fn>|2.1.4<space|2spc>Initalizing the X
      display<value|toc-dots><pageref|toc-9>>

      <with|left margin|<quote|1.5fn>|2.1.5<space|2spc>Initializing the
      T<rsub|<space|-0.4spc><move|<resize|<with|index
      level|<quote|0>|E>||||0.5fn|>|0fn|-0.1fn>><space|-0.4spc>X<rsub|<space|\
      -0.4spc><move|<resize|M<space|-0.2spc>A<space|-0.4spc>CS||||0.5fn|>|0fn\
      |-0.1fn>> server<value|toc-dots><pageref|toc-10>>

      <with|left margin|<quote|1.5fn>|2.1.6<space|2spc>Loading the initial
      buffer<value|toc-dots><pageref|toc-11>>

      <with|left margin|<quote|1.5fn>|2.1.7<space|2spc>The main event loop
      <with|font series|<quote|medium>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      series|<quote|medium>|font shape|<quote|slanted>|x_display_rep>><with|m\
      ode|<quote|math>|::><with|font series|<quote|bold>|math font
      series|<quote|bold>|<with|font family|<quote|rm>|font
      shape|<quote|right>|event_loop>> in <with|font
      family|<quote|tt>|Window/X/x_loop.gen.c><value|toc-dots><pageref|toc-12\
      >>
    </associate>
  </collection>
</auxiliary>
