<TeXmacs|0.3.5.14>

<style|<tuple|article|allouche>>

<\body>
  <\expand|make-title>
    <title|Porting <TeXmacs> to GTK+>

    <author|David Allouche>

    <expand|title-date|Around may 2001 (version 0.3.4.4)>
  </expand>

  <\rsection|Introduction>
    This document is a study of how <TeXmacs> may be ported to the GTK+
    system. It is based on various notes I took while studying the source.

    The basic problem I met when wondering how to port <TeXmacs> to another
    windowing toolkit was a chicken-and-egg type. GTK+ provides its own event
    loop which perform the event dispatching on behalf the application.
    However, this dispatching requires the user interface was built using
    GTK+. Conversely, GTK+ widgets rely on the event loop to do all kinds of
    black magic to behave properly. So, ideally, porting an application to
    GTK+ would require porting <em|simultaneously> the event loop and the
    user interface components. The problem is how to split the work in work
    units of manageable size?

    In order to get some insight on the guts of <TeXmacs> I started stepping
    through it and noting everything I found that related to the display. The
    first enlightenment I had was simply realizing that the author has
    <em|heavily> modularized its code (to the point it is sometime difficult
    to browse through). So great insight could hopefully be gained from
    examining and documenting the interfaces of the display related classes.

    <\rsection|Previous works>
      <\description>
        <expand|item*|<TeXmacs> notes:>my first study on <TeXmacs>. Talks
        mostly of the fundamentals of texmacs, especially how reference
        counting is implemented, which is far from trivial.

        <expand|item*|Step into <TeXmacs>:>the notes I took while executing
        <TeXmacs> step by step from the start.
      </description>
    </rsection>
  </rsection>

  <\rsection|The <type|display> interface>
    The <dir|Window/><file|display.gen.h> defines an abstract display
    interface. This interface is made of the abstract class
    <type|display_rep> and of three utility functions.

    The display implements such global behaviour as the event loop, the
    palette, loading of PostScript fonts, grabbing of pointer and keyboard,
    selection related messages, but also assorted things that could be
    defined elsewhere, like language support and help balloon setting.

    <\interface|Utility functions>
      <funcproto|display|open_display|<param|int|argc><param|char**|argv>>

      <\expand|/*>
        Create a <type|display> object and perform any required
        initialization. This function encapsulates the concrete type of the
        <type|display> object.
      </expand>

      <funcproto|void|close_display|<param|display|dis>>

      <\expand|/*>
        This function is the symetric of <expand|func|open_display>. It must
        release any resource reserved by the display.
      </expand>

      <funcproto|void|set_interpose_handler|<param|void(*)(void)|>>

      <\expand|/*>
        As the name says, sets the interpose handler. This handler is called
        in the main event loop after event polling and dispatching (that is
        when the event queue is finally empty) and before event
        post-processing. It is also called when the process is idle.
      </expand>
    </interface>

    <class|display_rep|<\members>
      <funcproto|void|get_extents|<param|SI&|width><param|SI&|height>>

      <\expand|/*>
        Gives the dimensions of the display (the screen). The lengths must be
        multiplied by the constant <const|PIXEL>.
      </expand>

      <funcproto|void|event_loop|>

      <\expand|/*>
        The usual outer loop of event driven program.

        The loop exits when there no longer any open window.

        It starts by processing all the pending X events, optionally it makes
        the process sleep (if it has been idle for some time). Then it calls
        the interpose handler, pops up the active help balloon if the balloon
        timer has expired, sends <expand|func|repaint_invalid_regions>
        messages to all windows, and handles pending alarms (decrease timer
        and execute if they expired).
      </expand>

      <funcproto|void|get_rgb|<param|color|col><param|int&|r><param|int&|g><p\
      aram|int&|b>>

      <\expand|/*>
        Get the RGB components for a given color number. This seems to imply
        that the display use pseudocolor, because it rely on proper
        initialization of the colormap for the mapping to be right.
      </expand>

      <funcproto|color|get_color|<param|string|s>>

      <\expand|/*>
        Return the color number for a given color name. If the color name is
        unknown, return black.
      </expand>

      <\noparskip>
        <funcproto|void|load_dictionnary|<param|string|name><param|string|fro\
        m><param|string|to>>

        <funcproto|void|set_output_language|<param|string|s>>

        <funcproto|string|get_output_language|>
      </noparskip>

      <funcproto|string|translate|<param|string|s><param|string|from><param|s\
      tring|to>>

      <\expand|/*>
        These methods appear quite mysterious now. Most of them appear merely
        to encapsulate the dictionnary implementation as a policy of the
        display. <expand|func|set_output_language>, however, can generate
        update events, but the scope of this language setting and its
        relation with document-specified languages is unclear.
      </expand>

      <funcproto|font|default_font|>

      <\expand|/*>
        Virtual constructor for the built-in default font used on this
        display. The default font can be affected by the language of the
        display: Cyrillic displays get Cyrillic fonts, while other displays
        get vanilla Computer Modern.\ 
      </expand>

      <funcproto|void|load_ps_font|<param|string|family><param|int|size><para\
      m|int|dpi><param|bitmap_metric&|bmm><param|bitmap_font&|bmf>>

      <\expand|/*>
        Computes metrics and bitmaps for a X font. I speculate that
        <type|bitmap_metric> and <type|bitmatp_font> are components of a
        PostScript font, thus the name. The font <var|size> must be given in
        typographic points. I think this part should be kept verbatim in a
        GTK+ implementation.
      </expand>

      <funcproto|void|grab_pointer|<param|widget|wid>>

      <\expand|/*>
        Perform some X Window black magic. Only used in
        <expand|func|<qmeth|basic_widget_rep|handle_mouse_grab>>, which in
        turn in only used in <qmeth|basic_widget_rep|handle> for
        <const|MOUSE_GRAB_EVENT>, which is only used in the X event
        dispatching system. I would bet that can be left a no-op in a GTK
        interface.
      </expand>

      <\noparskip>
        <funcproto|void|ungrab_pointer|>

        <funcproto|bool|has_grab_pointer|<param|widget|w>>

        <funcproto|void|grab_keyboard|<param|widget|wid>>

        <funcproto|void|ungrab_keyboard|>
      </noparskip>

      <\expand|/*>
        I did not look at them, but by their name one could guess the same
        thing can be said about them as about <expand|func|grab_pointer>.
      </expand>

      <\noparskip>
        <funcproto|tree|get_selection|<param|widget|wid><param|string|k>>

        <funcproto|bool|set_selection|<param|widget|wid><param|string|k><para\
        m|tree|t><param|string|s>>

        <funcproto|void|clear_selection|<param|string|key>>
      </noparskip>

      <\expand|/*>
        Interprocess communication via selections. Quite a hairy user
        interface issue. You would better have a look at the section
        <em|managing selections> from the GTK+ tutorial to understant what
        this is all about.

        Additionnally to performing IPC, these methods support internal
        TeXmacs selections. In <type|x_display_rep>, those selections are
        stored in the <var|selections> attribute wich is a <type|hashmap
        \<less\>string, tree\<gtr\>>. <expand|func|set_selection> and
        <expand|func|clear_selection> always update the <var|selections>
        hashmap. and <expand|func|get_selection> first tries to get the
        selection corresponding to the given key in this hashmap, and only
        use the X protocol as a fallback. These internal selections can be
        used in with the appropriate Scheme glue functions.

        If the selection key is not "primary" no attempt is made to use the X
        protocol. If there is no internal selection with this key,
        <expand|func|get_selection> returns "none". And, after updating the
        <var|selection> hashmap, <expand|func|set_selection> just returns
        <const|TRUE>, and <expand|func|clear_selection> returns.

        As a consequence of this internal selection caching, the X protocol
        is never used for cutting and pasting inside a given TeXmacs process.\
        \ 

        By experimentation, one can determine that when transferring
        selection with other processes, the selection is considered to be a
        TeXmacs tree, string encoded using the same format as in TeXmacs
        files. Code examination reveals that selection exporting is done in
        <qmeth|x_display_rep|procees_event> when handling
        <const|SelectionRequest> events. The provided targests are
        <const|AnyPropertyType> and <const|XA_STRING>, and both only return
        the <var|selection> attribute of <type|x_display_rep>.

        The <type|string> <var|k> is the selection identifier.
      </expand>

      <funcproto|void|delayed_message|<param|widget|wid><param|string|mess><p\
      aram|time_t|delay>>

      <\expand|/*>
        Register a <type|message> in the delayed message queues. When the
        <var|delay> has expired (<var|delay> is counted relative to the time
        of the calling of <expand|func|delayed_message>), <var|wid> will be
        sent the <type|event> built from <var|mess> and the time of
        expiration of the message.

        In TeXmacs, time is always counted in miliseconds.

        Events are sent using the overload
        <operproto|event|\<less\>\<less\>|<param|widget|><param|event|>> from
        the main event loop. In turn, the <operproto||\<less\>\<less\>|> just
        passes the <type|event> to the <expand|func|handle> method of the
        <type|widget>.

        One simple way to implement this would be with GTK+ time-outs. Each
        delayed message would be associated with one time-out, and the
        time-out handler would return <const|FALSE> (destroy time-out).
      </expand>
    </members>>
  </rsection>

  <\rsection|The <type|event> interface>
    This interface (<path|Window|event.gen.h>) defines a hollow <type|event>
    type whose only feature is a <type|int> <var|type> member and a number of
    utility functions.

    All these utility functions are wrappers for constructors of the
    corresponding concrete event representation. They all return a banalized
    <type|event> object, and they are defined in the diverse files in
    <dir|Window/Event> (for basic, composite, and scroll events) and in
    <path|Window/Widget/Input|input_widget.gen.cc> for the input events.

    Useful information about the event implementation can be found in the
    "Graphical interface" online help file of TeXmacs. For the moment, this
    looks quite complicated and of unknown relevance to a port to GTK+. Let's
    have a look at the window interface first
  </rsection>

  <\rsection|Considering GDK>
    Maybe a rational approach to the porting problem would involve first a
    switch from plain X to GDK. This would allow to keep most of the user
    interface untouched, as GDK is very close to X, while increasing
    portability.

    Porting to Win32 might be simply a question of porting to GDK and linking
    with Cygwin runtimes, or hopefully with any required native libraries.

    In order to actually do this, one must first define what parts of the
    code actually depend on X.
  </rsection>

  <\rsection|Tracking X11 dependances>
    All X11 includes are in <path|Window/X|x_display.gen.h> in the
    <module|x_display> module.

    <\itemize>
      <item><file|x_display.gen.h> is included in
      <path|Window/X|x_drawable.gen.h>

      <item><file|x_drawable.gen.h> is included in
      <path|Window/X|x_window.gen.h>

      <item><file|x_window.gen.h> is included in several <with|font
      family|tt|gen.cc> files in <dir|Window/X>

      <item>The module <module|x_display> is imported in <module|x_drawable>.
      The module <module|x_drawable> is imported in <module|x_window>. Wich
      in turn is imported in several code modules defined in <dir|Window/X>.
    </itemize>

    Good, so porting to GDK will only require examination of files in
    <dir|Window/X>.
  </rsection>

  <\rsection|Modifying the build system>
    Updating the TeXmacs build system to properly detect and use GDK is
    non-trivial. Essentially it all boils down to the fact that TeXmacs build
    system is highly non standard.

    <\rsection|Updating the <sh|configure> script>
      The <with|font family|tt|<sh|configure>> script must be updated to
      properly check for the presence of GTK developement files. Firstly one
      must update the <file|configure.in> file to include the following
      script:

      <\verbatim>
        dnl ******************************

        dnl gtk+ checking

        dnl ******************************

        AM_PATH_GTK(1.2.7,,

        \ \ AC_MSG_ERROR([

        *** GTK+ 1.2.7 or better is required. The latest version of GTK+

        *** is always available from ftp://ftp.gtk.org/.]),

        \ \ \ \ )
      </verbatim>

      Then, the <file|aclocal.m4> file must be updated. Since the Guile
      related script is handmade, this requires some fiddling. First rename
      the old <file|aclocal.m4> file (to <file|aclocal.m4-old> for example),
      then run <sh|aclocal> to generate a new <file|aclocal.m4>, finally
      replace the part related to Guile in the generated <file|aclocal> by
      the code in the old <file|aclocal> file.

      Additional tweaks must be done to the <file|aclocal.m4> file. Namely,
      any occurance of the <with|font family|tt|CFLAG> and <with|font
      family|tt|ac_save_CFLAGS> variable names must be replaced by <with|font
      family|tt|CXXFLAGS> and <with|font family|tt|ac_save_CXXFLAGS>. This is
      required because using a C++ compiler will make configure ignore the
      <with|font family|tt|CFLAGS> variable when compiling test programs, so
      the GTK+ test program will fail to compile.
    </rsection>

    <\rsection|Updating the builder>
      Here comes the really non standard stuff. Usually, the next step would
      have been to update the <file|makefile.in> of the appropriate source
      subtree to enable linking against GDK. However there is no such thing.
      Instead, there is a <file|<file|window.gen.make>> which is used by
      <sh|gencc>.

      Here, one will notice the use of the special <sh|gencc> option
      <with|font family|tt|+X-Window>, which enables compiling and linking of
      the specified object against the X Window libraries. Obviously
      interfacing with the configure system is done in the <sh|gencc> source
      tree.

      Special options are handled in the the <file|gg_system.in> file in the
      <expand|func|special_object_option>,
      <expand|func|special_library_option> and
      <expand|func|special_binary_binary> functions. There the special
      options are mapped to shell variables whose name begin in <with|font
      family|tt|CXX>, <with|font family|tt|LD> and <with|font family|tt|LDB>.

      Further examination revealed that the <with|font family|tt|+X-Window>
      is also used in any library that uses the <with|font family|tt|window>
      object file as well as in the <file|texmacs.bin> executable file. This
      sets an interesting problem: <strong|how to elegantly choose at compile
      time between the GDK and X Window system linking?>

      The system dependant shell variables to which <sh|gencc> special
      options are ultimately resolved are set by the <sh|configure> script in
      the <path|gencc/src|gg_system.in> file.

      Therefore the next step is to update <file|gg_system_in> to define
      <const|CXXGTK>, <const|LDGTK>, <const|LDBGTK>, and to update the
      <expand|func|special_object_option>,
      <expand|func|special_library_option> and
      <expand|func|special_binary_binary> functions to recognize the
      <with|font family|tt|+gtk> special option.

      Then, the <file|*.make.gen> files using the <with|font
      family|tt|+X-Window> option must be modified to use the <with|font
      family|tt|+gtk> option.
    </rsection>
  </rsection>

  <\rsection|Porting TeXmacs>
    Here we come. Eventually starting to put my hands on the code.

    I start by the beginning, first porting the <file|x_init.gen.cc> code. I
    decided to be has thorough as possible, by first commenting all the code
    out, then only uncommenting code that has been appropriately understood
    and modified.

    Generally, I noticed that a significant portion of the code needs not be
    modified, since it is effectively independant of the underlying display
    implementation. I guess it should be separated appropriately, but I
    decided not to do it to not get distracted of the porting work.

    I found a strange thing: the <expand|func|get_xmodmap> function
    initializes a set of static variables, but they appear to never get used.
    Moreover, its code appears to be non-portable. \ Good. I just put it
    away.
  </rsection>
</body>

<\initial>
  <\collection>
    <associate|preamble|false>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page medium|papyrus>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|page type|a4>
    <associate|reduction page bottom margin|15mm>
    <associate|even page margin|30mm>
    <associate|reduction page left margin|25mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-10|<tuple|6|?>>
    <associate|toc-11|<tuple|6.1|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-12|<tuple|6.2|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-13|<tuple|7|?>>
    <associate|toc-3|<tuple|2|?>>
    <associate|toc-14|<tuple|8|?>>
    <associate|toc-4|<tuple|2.1|?>>
    <associate|toc-15|<tuple|7|?>>
    <associate|toc-5|<tuple|2.2|?>>
    <associate|toc-6|<tuple|2.2.1|?>>
    <associate|toc-7|<tuple|3|?>>
    <associate|toc-8|<tuple|4|?>>
    <associate|toc-9|<tuple|5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font series|<quote|bold>|1<space|2spc>Introduction><\
      value|toc-dots><pageref|toc-1><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|2<space|2spc>Previous
      works><value|toc-dots><pageref|toc-2><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|3<space|2spc>The <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|rm>|font series|<quote|medium>|font
      shape|<quote|slanted>|display>> interface><value|toc-dots><pageref|toc-\
      3><vspace|0.5fn>

      3.1<space|2spc>Utility functions<value|toc-dots><pageref|toc-4>

      3.2<space|2spc>Display methods<value|toc-dots><pageref|toc-5>

      <vspace*|1fn><with|font series|<quote|bold>|4<space|2spc>The <with|font
      series|<quote|medium>|math font series|<quote|bold>|<with|font
      family|<quote|rm>|font series|<quote|medium>|font
      shape|<quote|slanted>|event>> interface><value|toc-dots><pageref|toc-6>\
      <vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|5<space|2spc>Considering
      GDK><value|toc-dots><pageref|toc-7><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|6<space|2spc>Tracking X11
      dependances><value|toc-dots><pageref|toc-8><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|7<space|2spc>Modifying the
      build system><value|toc-dots><pageref|toc-9><vspace|0.5fn>

      7.1<space|2spc>Updating the <with|font family|<quote|tt>|configure>
      script<value|toc-dots><pageref|toc-10>

      7.2<space|2spc>Updating the builder<value|toc-dots><pageref|toc-11>

      <vspace*|1fn><with|font series|<quote|bold>|8<space|2spc>Porting
      TeXmacs><value|toc-dots><pageref|toc-12><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>
