<TeXmacs|0.3.5.14>

<style|<tuple|article|allouche>>

<\body>
  <\expand|make-title>
    <\title>
      Handling of cursor motion
    </title>

    <title|<vspace*|0.5fn>and creation of boxes>

    <author|David Allouche>

    <expand|title-date|March 2002 (version 0.3.5.14)>
  </expand>

  <rsection|Introduction|This document is made of notes I have written down
  while trying to fix a bug related to visual motion commands with
  multicolumn blocks.>

  <\rsection|Vertical motion>
    Cursor motion commands are implemented in <type|edit_cursor_rep> in
    <file|Edit/Interface/>.

    In vertical motion commands, the cursor should be moved even if there no
    box just above/under the current cursor position. In such cases, the
    cursor should be moved to the right most box on the left of the cursor in
    the line above.

    The public methods handling vertical cursor motion are:

    <\itemize>
      <item><methproto|void|edit_cursor_rep|go_up|>

      <item><methproto|void|edit_cursor_rep|go_down|>
    </itemize>

    Both are based on the protected method

    <\itemize>
      <item><methproto||edit_cursor_rep|cursor_move|<param|SI|dx><param|SI|dy\
      >>
    </itemize>

    which, in turn, only calls

    <\itemize>
      <item><methproto||edit_cursor_rep|cursor_move_sub|...>
    </itemize>

    with the reference parameters set to the appropriate field of the
    <em|ghost cursor>, <keyw|this><apply|oper|-\<gtr\>><var|mv>.

    Debugging proved that the problem is not in this function, but instead in
    <funcproto||tree_path|<param|SI|><param|SI|><param|SI|>>. Indeed, the
    ability to find a box, either by motion or by click is affected by the
    width of boxes <em|farther> from the initial position.

    <funcproto||tree_path|<param|SI|><param|SI|><param|SI|>>, essentially
    just calls the <funcproto||find_tree_path|<param|SI|><param|SI|><param|SI\
    |>> of the <var|eb> data member. and perform adjustements on the found
    path (supposedly to avoid uneditable paths).
  </rsection>

  The problem appeared to be that, when a page contains 1 column and 2
  columns page items, the 1 column box only have the width of its contents,
  and not the width of the page. The visible consequence is that
  <funcproto||find_tree_path|<param|SI|><param|SI|><param|SI|>> can fail to
  find the one column items.

  <\rsection|Click position>
    Similarly to vertical motion, the position set by a mouse click should
    fall back to the nearest box on the left of the cursor in case of failure
    to find a box at the clicked point.

    The function handling motion to pixel commands is:

    <\itemize>
      <item><methproto|void|edit_cursor_rep|go_to|<param|SI|x><param|SI|y>>
    </itemize>
  </rsection>

  <\rsection|Production of boxes>
    Here we will discuss the process of typesetting as it can be traced for a
    simple document. The focus will be on how main boxes are created, other
    processing (<abbr|esp.> environment) and management of <em|surround>
    boxes will not be detailed.

    <\rsection|Production of paragragh boxes>
      Essentially, typesetting is performed by a recursive descent in the
      <type|bridge>s, starting with the instance associated to the document
      root. The root <type|bridge> contains at least (maybe exactly, I have
      not checked) one child bridge for every paragraph in the document.

      While descending the tree of <type|bridges> through recursive calls to
      <methproto||bridge_rep|typeset|<param|int|>>, <em|with> elements do not
      perform any action excepted updating and restoring the environment.
      Elements containing multiple paragraphs are used as <em|Decoration>s
      (Design Pattern) for a nested <type|bridge_document_rep>.

      Once we have reached the paragragh level, the
      <methproto||typesetter_rep|insert_paragraph|<param|tree|t><param|path|i\
      p>> (or <expand|func|insert_parunit>) is called with the appropriate
      typesetter state. In turn, this function will call
      <expand|func|typeset_stack>, this function will create a
      <type|lazy_paragraph> and put in the current <em|surround>s and the
      paragrah contents as an <template|array|<type|line_item>>.

      The method that will do the work is
      <methproto||concater_rep|typeset|<param|tree|><param|path|ip><param|boo\
      l|active_flag>>. The <type|line_item>s will contain numerous boxes
      (like those created by <methproto||concater_rep|typeset_substring|<para\
      m|string|><param|path|ip><param|int|pos>>) but we are not concerned by
      such a level of detail. Just after, the
      <methproto||concater_rep|finish|> method will perform various second
      pass processing on the <em|concat> that are not possible to perform
      locally.

      Back to <expand|func|typeset_stack>, the just filled
      <type|lazy_paragraph> is sent a <funcproto||format_paragrah|> message
      which will perform global paragraph formatting like line breaking. Then
      various formatting operations are done that we will not detail.

      After typesetting paragraph level data, we are out of
      <methproto||bridge_rep|my_typeset|> and back to
      <methproto||bridge_rep|typeset|<param|int|>>. You remember? The
      function that performs the recursive descent through bridges... So, we
      were typesetting the first paragraph of the multicolumn block.
      Actually, it seems that the box have not yet be multicolumnized,
      experimentation suggest they will not be before a paragraph with a
      different number of columns is stacked (or a page break occur). The
      typesetting of the paragraph has provided us with an
      <template|array|<type|page_item>>. Since this was our first paragraph,
      it is simply <expand|func|insert_stacked> by the typesetter.

      Back to the <methproto||bridge_with_rep|typeset|<param|int|>> method
      something weird happen. A 7 elements <template|array|<type|page_item>>
      is sent to the typesetter via <expand|func|insert_stacked>. Those
      elements were stacked in <expand|func|my_typeset> in the local
      typesetter state provided by the use of <expand|func|local_start> and
      <expand|func|local_end> methods.

      Back to the <methproto||bridge_document_rep|my_typeset|<param|int|>>
      method, the one of the root bridge. We are going to typeset the single
      column paragraph, so shit should happen soon.

      We now descend in the <expand|func|typeset> method of the bridge for
      the second block of the document. Since local typesetting of the block
      happen in a supposedly empty typesetter thanks to <expand|func|local_*>
      methods, we did not step through the <expand|func|my_typeset> method.
      Actually, nothing interesting seemed to happen before...
    </rsection>

    <\rsection|Typesetting pages>
      We are back to <methproto||typesetter_rep|typeset|>. At this point, we
      have a <template|array|<type|page_item>> whose components have already
      been stacked. However we did not see anything multicol specific happen
      and parapraph level boxes have not (as far as we remember) been
      created. Now, a <type|pager> is created. In the constructor, various
      environment properties are fetched, including the <var|text_width>
      (424648).

      Then <methproto||pager_rep|make_pages|> calls the appropriate paging
      method for the media type, here, <expand|func|papyrus_make>. There, we
      will process the <template|array|<type|page_item>> produced by the
      typesetting of the <type|bridges>. The
      <template|array|<type|page_item>> contains one item for every line of
      type, the pager is going to arrange these in multicols and regular
      paragraphs.

      Once the <expand|func|break_pages> function is done we have a 1 item
      <type|skeleton> (single page, so only one <type|pagelet>) containing
      two <type|insertion>s,\ 

      <\itemize>
        <item>one for the multicol\ 

        <\itemize>
          <item><var|type> is a tree labeled by <verbatim|"multi-column">

          <item>its <type|skeleton> contains two <type|pagelet>s, likely one
          for each column
        </itemize>

        <item>one for the normal paragraph

        <\itemize>
          <item><var|type> is an empty <verbatim|STRING> node

          <item>its skeleton is empty
        </itemize>
      </itemize>
    </rsection>

    \;
  </rsection>

  \;
</body>

<\initial>
  <\collection>
    <associate|page right margin|5mm>
    <associate|dpi|600>
    <associate|reduction page left margin|25mm>
    <associate|language|english>
    <associate|page width|998400unit>
    <associate|odd page margin|5mm>
    <associate|paragraph width|150mm>
    <associate|page medium|automatic>
    <associate|shrinking factor|5>
    <associate|page top margin|5mm>
    <associate|reduction page right margin|25mm>
    <associate|reduction page bottom margin|15mm>
    <associate|page type|a4>
    <associate|even page margin|5mm>
    <associate|page bottom margin|5mm>
    <associate|page height|1209600unit>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-5|<tuple|4.1|?>>
    <associate|toc-6|<tuple|4.2|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|idx-1|<tuple|1|?>>
    <associate|toc-2|<tuple|2|?>>
    <associate|toc-3|<tuple|3|?>>
    <associate|toc-4|<tuple|4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font series|<quote|bold>|1<space|2spc>Vertical
      motion><value|toc-dots><pageref|toc-1><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|2<space|2spc>Click
      position><value|toc-dots><pageref|toc-2><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|3<space|2spc>Production of
      boxes><value|toc-dots><pageref|toc-3><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>
